const express = require('express')
const app = express()
const port = 3001
const crypto = require('crypto')
const base64 = require('base-64')
const slack = require('../../slack')
const { game: { users } } = require('./utils')

const apiGetUserId = hash => {
  return Object.entries(userGetter.users)
    .filter(([id, user]) => user.pwHash === hash)
    .map(([id, user]) => id)[0]
}

const makeHash = pw =>
  crypto.createHash('md5')
    .update(pw)
    .digest('hex')

const addCommand = ({ commandNames, helpText, action, condition, hidden }) => {
  const route = async (req, res) => {
    const say = async msg => res.send(msg)
    try {
      const words = ['', ...Object.keys(req.query)]
      console.log('INCOMING API CALL:', name, words)
      const encoded = req.header('Authorization').substring(5)
      const decoded = base64.decode(encoded).substring(1)
      const event = {
        user: apiGetUserId(makeHash(decoded))
      }
      if (!event.user) {
        res.status(400)
        res.send(
          'User does not exist, or does not have a password.\n' +
          'See \'!setpw help\' for assistance.'
        )
        console.log('  bad password')
        return
      }
      const lastCall = userGetter.users[event.user].lastApiCall || 0
      const secondsBetweenCalls = 5
      const currentTime = Math.floor(new Date().getTime() / 1000)
      if (lastCall + secondsBetweenCalls > currentTime) {
        res.status(400)
        res.send(`Must have at least ${secondsBetweenCalls}s between api calls`)
        console.log('  rate limited')
        return
      }
      console.log(`  went through for ${slack.ourUsers[event.user]}`)
      userGetter.users[event.user].lastApiCall = currentTime

      await action({event, say, words})
    } catch (e) {
      console.error(e)
      await say(e.stack)
    }
  }
  commandNames.forEach(name =>
    app.get('/' + name.replace(/!/gi, ''), route)
  )
}

module.exports = {
  addCommand,
  makeHash,
  launch: () => app.listen(port, () => {
    console.log(`Express listening on port ${port}`)
  })
}